con
  RJOYLR = 0
  RJOYUD = 1
  LJOYLR = 2
  LJOYUD = 3
  RSLIDER = 4
  LSLIDER = 5

  DIYPACKETSIZE = 8

' XBee packets
    XBEE_TRANS_READY   =                     $01    ' Transmitter is ready for requests.
        ' Optional Word to use in ATDL command
    XBEE_TRANS_NOTREADY =             $02   ' Transmitter is exiting transmitting on the sent DL
        ' No Extra bytes.
    XBEE_TRANS_DATA     =                    $03    ' Data Packet from Transmitter to Robot*
        ' Packet data described below.  Will only be sent when the robot sends
        ' the remote a XBEE_RECV_REQ_DATA packet and we must return sequence number
    XBEE_TRANS_NEW      =                    $04    ' New Data Available
        ' No extra data.  Will only be sent when we are in NEW only mode
    XBEE_ENTER_SSC_MODE =                    $05    ' The controller is letting robot know to enter SSC echo mode
        ' while in this mode the robot will try to be a pass through to the robot. This code assumes
        ' cSSC_IN/cSSC_OUT.  The robot should probalby send a XBEE_SSC_MODE_EXITED when it decides
        ' to leave this mode...        
        ' When packet is received, fPacketEnterSSCMode will be set to TRUE.  Handlers should probalby
        ' get robot to a good state and then call XBeeHandleSSCMode, which will return when some exit
        ' condition is reached.  Start of with $$<CR> command as to signal exit
    XBEE_REQ_SN_NI      =                    $06    ' Request the serial number and NI string

'[Packets sent from Robot to remote]
    XBEE_RECV_REQ_DATA  =                    $80    ' Request Data Packet*
        ' No extra bytes: expect to receive XBEE_TRANS_DATA_PACKET
    XBEE_RECV_REQ_NEW   =                    $81    ' Request Only New data
        ' No Extra bytes goes into New only mode and we will typically 
        ' wait until Remote says it has new data before asking for data.
        ' In new mode, the remote may choose to choose a threshold of how big a change
        ' needs to be before sending the XBEE_TRANS_NEW value.
    XBEE_RECV_REQ_NEW_OFF =          $82    ' We will request data when we want it
    XBEE_RECV_NEW_THRESH  =          $83    ' Set new Data thresholds
        ' currently not implemented
    XBEE_RECV_DISP_VAL    =                  $84    ' Display a value on line 2
        ' If <cbExtra> is  0 then we will display the number contained in <SerialNumber> 
        ' If not zero, then it is a count of bytes in a string to display.
    XBEE_RECV_DISP_STR    =                  $85    ' Display a string value on line 2
    XBEE_PLAY_SOUND       =                  $86    ' Will make sounds on the remote...
        '      <cbExtra> - 2 bytes per sound: Duration <0-255>, Sound: <Freq/25> to make fit in byte...
    XBEE_SSC_MODE_EXITED  =          $87    ' a message sent back to the controller when
        ' it has left SSC-mode.
    XBEE_SEND_SN_NI_DATA  =          $88    ' Response for REQ_SN_NI - will return
        ' 4 bytes - SNH
        ' 4 bytes - SNL
        ' up to 20 bytes(probably 14) for NI
  
        CXBEEPACKETTIMEOUTMS  = 250                                       'how long to wait for packet after we send request
        CXBEEFORCEREQMS       = 1000                                    ' if nothing in 1 second force a request...
        CXBEETIMEOUTRECVMS    = 2000                                    ' 2 seconds if we receive nothing

OBJ               
    XBEE : "XBee_Object_1"       
    DBG   : "SerialMirror"

var
  byte _fNewOnlyMode
  word _wCurDL
  byte  _fPacketValid
  byte  _fPacketValidPrev

  byte  _fPacketTimeOut
  byte  _fPacketForced
  byte  _fReqDataForced
  byte _fTransReadyRecvd
  byte _fReqDataPacketSent
  long  _tcLastPacket 
  long  _tcLastRequest

  byte  _bPacketVal[6]           ' packet data
  word  _wButtons               ' buttons
  word  _wButtonsPrev             ' Previous buttons
  byte  _fPacketChanged          ' did the packet change???  
  byte  _bFrameNum


pub start(rx, tx, mode, baud) 
    XBEE.start(rx,tx,mode,baud)           ' XBee Comms - RX,TX, Mode, Baud

    'XBEE.AT_Init                     ' Initialize for fast AT command use - 5 second delay to perform
    XBEE.AT_Config(string("ATAP 1")) ' Tell system we are going into config mode...
'    Value := XB.RxDec              ' accept a decimal value.

'   Add some debug code
    XBEE.API_Query(string("MY"))                        ' issue the my command
    XBEE.API_RxTime(100)
    DBG.str(string("XBee MY: "))
    DBG.hex(XBEE.rxValue ,8)
    dbg.tx(13)
    _bFrameNum := 1
  _fNewOnlyMode := 0
pub stop
  XBEE.Stop

pub rxFlush

pub SetDL(wNewDL)
  _wCurDL := wNewDL

pub PacketChanged
  return _fPacketChanged

pub PacketForced
  return _fPacketForced
  
pub IButtonPressed | iBtn
  if _wButtons
    repeat iBtn from 0 to 15
      if Button(iBtn)
        return iBtn
  return -1
  
pub Button(iBtn)
  if (_wButtons & (1<<iBtn)) and ((_wButtonsPrev & (1<<iBtn))== 0)
    return true
  return false

pub GetValue(iValue)
  return _bPacketVal[iValue]

pub SendNewDataOnlyPacket(fNewOnly)
    if (fNewOnly)
        SendPacket(_wCurDL, XBEE_RECV_REQ_NEW,  0, 0) 
    else
        SendPacket(_wCurDL, XBEE_RECV_REQ_NEW_OFF,  0, 0) 

    _fNewOnlyMode := fNewOnly

pub OutputVal(w)
    SendPacket(_wCurDL, XBEE_RECV_DISP_VAL, 2, @w)

pub OutputString(strptr)
    SendPacket(_wCurDL, XBEE_RECV_DISP_STR, STRSIZE(strptr), strptr)

pub PlaySounds(dataptr) | cNotes
    cNotes := byte[dataptr++]
    SendPacket(_wCurDL, XBEE_PLAY_SOUND, cNotes*2, dataptr)


pub SendPacket(wDL, bPHType, cbExtra, pbExtra) | PacketPtr, PO

  _bFrameNum++
  if _bFrameNum == 0
    _bFrameNum := 1            ' dont allow 0 for packet number...
  XBEE.SetFrameID(_bFrameNum)

  if cbExtra== 0
      XBEE.API_Tx(wDL, bPHType)        ' only need to send the one byte...
  else
'      DBG.dec(cbExtra)
'      DBG.tx(" ")
'      DBG.hex(pbExtra, 8)
      PO := XBEE.API_Packet    ' we will use the packet storage in the XBee module...
      packetPtr := 0
      byte[PO + packetPtr] := bPHType
      packetPtr++
      Repeat cbExtra                 ' Add string to packet
        byte[PO + packetPtr] := byte[pbExtra++]
        packetPtr++
      XBEE.API_Array(wDL, PO, packetPtr)               ' output

'==============================================================================
' [ReceiveXBeePacket] - This function will try to receive a packet of information
'         from the remote control over XBee.
'
' the data in a standard packet is arranged in the following u8 order:
'    0 - Buttons High
'    1 - Buttons Low
'     2 - Right Joystick L/R
'    3 - Right Joystick U/D
'    4 - Left Joystick L/R
'    5 - Left Joystick U/D
'     6 - Right Slider
'    7 - Left Slider
'==============================================================================
pub ReceivePacket  | _fNewPacketAvail, DataPtr, i, tcCurrentTime, ulTimeDiffMS

    _fNewPacketAvail := false

    _fPacketValidPrev := _fPacketValid    ' Say the data is in the same state as the previous call...
    _fPacketValid := false
    _fPacketTimeOut := false
    _fPacketForced := false

    _wButtonsPrev := _wButtons                          ' save away the old buttons...
    '    We will first see if we have a packet header waiting for us.
    '  BUGBUG:: Question should I loop after I process a package and only get out of the
    '             loop when I have no more data, or only process one possible message?
    '            Maybe depends on message?
    repeat

      XBEE.API_RxTime(50)                            ' Try to read in a packet...
             ' Based on RxIdent (type of packet), displays information
      if XBEE.rxIdent == $ff
        quit ' break out of the repeat loop.
      elseif XBEE.rxIdent == $81
      
        DataPtr := XBEE.rxdata
       
'        DBG.str(string("RP: "))
'        DBG.hex(byte[DataPtr],2)
'        DBG.tx(",")
'        DBG.dec(XBee.RXLen)
'        DBG.tx(13)
       
        if (byte[DataPtr] == XBEE_TRANS_DATA) and  (XBEE.RXLen > DIYPACKETSIZE)
        '-----------------------------------------------------------------------------
        ' [XBEE_TRANS_DATA]
        '-----------------------------------------------------------------------------
            _wButtons := byte[DataPtr+1] + (byte[DataPtr+2] << 8)
            _fPacketChanged := _wButtons <> _wButtonsPrev
            repeat i from 0 to 5
              if _bPacketVal[i] <> byte[DataPtr+3 + i]
                _fPacketChanged := true
                _bPacketVal[i] := byte[DataPtr+3 + i]           
            _fPacketValid := true    ' data is valid
            _fPacketForced := _fReqDataForced    ' Was the last request forced???
            _fReqDataForced := 0                ' clear that state now
            _tcLastPacket := cnt

'           if _fPacketChanged
'               DBG.str(string("DP: "))
'               dbg.hex(_wButtons, 4)
'               repeat i from 0 to 5
'                   dbg.tx(" ")
'                   dbg.hex(_bPacketVal[i], 2)
'               dbg.tx(13) 
            return true    '          ' get out quick!
          
       
        elseif (byte[DataPtr] == XBEE_TRANS_READY) and (XBEE.RXLen == (2+1))
            '-----------------------------------------------------------------------------
            ' [XBEE_TRANS_READY]
            '-----------------------------------------------------------------------------
            _wCurDL := (byte[DataPtr+1]<<8) + byte[DataPtr+2]
            _fTransReadyRecvd := true        ' OK we have received a packet saying transmitter is ready.    
       
            DBG.str(string("Ready: "))
            DBG.hex(_wCurDL,4)
            DBG.tx(13)
             
            ' And tell the remote to go into New data only mode.
            SendNewDataOnlyPacket(1)
            _tcLastPacket := cnt
            _fReqDataPacketSent := 0                            ' make sure we don't think we have any outstanding requests
            _fNewPacketAvail := true                                    ' and try to get the first packet of data
        elseif (byte[DataPtr]  == XBEE_TRANS_NOTREADY) 
            '-----------------------------------------------------------------------------
             ' [XBEE_TRANS_NOTREADY]
            '-----------------------------------------------------------------------------
            DBG.str(string("Not Ready: "))
           _fTransReadyRecvd := 0            ' Ok not valid anymore...
       
        elseif (byte[DataPtr] == XBEE_TRANS_NEW) 
            '-----------------------------------------------------------------------------
            ' [XBEE_TRANS_NEW]
            '-----------------------------------------------------------------------------
'            DBG.str(string("XBEE_TRANS_NEW",13))
            _fNewPacketAvail := true
       
      '-----------------------------------------------------------------------------
    ' Exited above loop now See if we need to request data from the other side
    '-----------------------------------------------------------------------------
    ' Only send when we know the transmitter is ready.  Also if we are in the New data only mode don't ask for data unless we have been 
    '    old there
    ' is new data. We relax this a little and be sure to ask for data every so often as to make sure the remote is still working...
    ' 
    if _fTransReadyRecvd 
        tcCurrentTime := cnt

        ' Time in MS since last packet
        ulTimeDiffMS := (tcCurrentTime - _tcLastPacket)/(clkfreq/1000)  ' need to convert from us to ms...
        ' See if we exceeded a global timeout.  If so let caller know so they can stop themself if necessary...
        if (ulTimeDiffMS > CXBEETIMEOUTRECVMS) 
            _fPacketValid := 0
            _fPacketForced := true
            DBG.str(string("TO",13))
            _fTransReadyRecvd := false
            return false

        ' see if we have an outstanding request out and if it timed out...
        if _fReqDataPacketSent
            if ((tcCurrentTime-_tcLastRequest) > CXBEEPACKETTIMEOUTMS) 
                ' packet request timed out, force a new attempt.
                _fNewPacketAvail := true        ' make sure it requests a new one    

        ' Next see if it has been too long since we received a packet.  Ask to make sure they are there...
        if (!_fNewPacketAvail and (ulTimeDiffMS > CXBEEFORCEREQMS)) 
            _fNewPacketAvail := true
            _fReqDataForced := true        ' remember that this request was forced!

        if (!_fNewOnlyMode or (_fNewOnlyMode and _fNewPacketAvail)) 
            ' Now send out a prompt request to the transmitter:

'            DBG.str(string("SD REQ Data",13))
            SendPacket (_wCurDL, XBEE_RECV_REQ_DATA, 0, 0)    ' Request data Prompt (CmdType, ChkSum, Packet Number, CB extra data)
            _fReqDataPacketSent := true                                     ' yes we have already sent one.
            _tcLastRequest := tcCurrentTime                        ' remember when we sent this...
        _fPacketValid := _fPacketValidPrev    ' Say the data is in the same state as the previous call...
    return _fPacketValid