{{
Project Lynxmotion Phoenix
Description: Phoenix, control file.
  The control input subroutine for the phoenix software is placed in this file.
  Can be used with V2.0 and above
Configuration version: V1.0
Hardware setup: PS2

Date: Nov 17, 2010

       Walk method 1:
       - Left Stick    Walk/Strafe
       - Right Stick   Rotate

       Walk method 2:
       - Left Stick    Disable
       - Right Stick   Walk/Rotate


PS2 CONTROLS:
       [Common Controls]
       - Start         Turn on/off the bot
       - L1            Toggle Shift mode
       - L2            Toggle Rotate mode
       - Circle        Toggle Single leg mode
       - Cross         Toggle GP Mode
       - Square        Toggle Balance mode
       - Triangle              Move body to 35 mm from the ground (walk pos) 
                                       and back to the ground
       - D-Pad up              Body up 10 mm
       - D-Pad down    Body down 10 mm
       - D-Pad left    decrease speed with 50mS
       - D-Pad right   increase speed with 50mS

       [Walk Controls]
       - select                Switch gaits
       - Left Stick    (Walk mode 1) Walk/Strafe
                                       (Walk mode 2) Disable
       - Right Stick   (Walk mode 1) Rotate,           
                                       (Walk mode 2) Walk/Rotate
       - R1                    Toggle Double gait travel speed
       - R2                    Toggle Double gait travel length

       [Shift Controls]
       - Left Stick    Shift body X/Z
       - Right Stick   Shift body Y and rotate body Y

       [Rotate Controls]
       - Left Stick    Rotate body X/Z
       - Right Stick   Rotate body Y   

       [Single leg Controls]
       - select                Switch legs
       - Left Stick    Move Leg X/Z (relative)
       - Right Stick   Move Leg Y (absolute)
       - R2                    Hold/release leg position

       [GP Player Controls]
       - select                Switch Sequences
       - R2                    Start Sequence


NEW IN V1.0
 - First Release


}}

OBJ
  PS2   : "PS2_controller_new"       
  DBG   : "SerialMirror"
  CFG   : "Phoenix_Config"

VAR
'=============================================================================
' Global - Local to this file only...
'=============================================================================

  ' Lets define the variables that we will set as part of the control object
  byte _fHexOn                  ' Is the Robot supposed to be on/Active?   

  byte _GaitType                 ' Wich Gait is active
  byte _BalanceMode              ' Balance mode
  word _InputTimeDelay           ' Desired Input Time delay generated from the inputs...
  word _SpeedControl

' Settings for position and rotation of body 
  long _BodyPosX
  long _BodyPosY
  long _BodyPosZ
  long _BodyRotX1
  long _BodyRotY1
  long _BodyRotZ1
' Settings for how far to move or rotate and left the leg.
  long _TravelLengthX
  long _TravelLengthZ
  long _TravelRotationY
  
  word _LegLiftHeight
  byte DoubleHeightOn
  byte DoubleTravelOn
  byte WalkMethod
  
' GP Player settings... Will need info as well from main object...
  byte  _GPSeq         ' Which Sequence
  byte  _GPStart       ' should we start
  byte _fGPEnable      ' need to set as part of the init...

' SingleLegMode

  byte  _SelectedLeg     
  byte _fSLHold
  long _SLLegX
  long _SLLegZ
  long _SLLegY
  
' Not sure about these...
  word    _BodyYOffset
  byte    _BodyYShift 

' Non exported values...

  byte    bControlMode       ' Our current control mode...  


DAT     
 
        SND_BEEP        byte    1, 50, 2000/25
        SND_BEEP2       byte    2, 50, 2000/25, 50, 2500/25
        SND_HEXON       byte    3, 60, 2000/25, 80, 2250/25, 100, 2500/25
        SND_HEXOFF      byte    3, 60, 2500/25, 80, 2250/25, 100, 1000/25

con
  WALKMODE      =    0
  TRANSLATEMODE =    1
  ROTATEMODE    =    2
  SINGLELEGMODE =    3
  GPPLAYERMODE  =    4

  GPSTARTED     = 0    ' Used in the GPSetState function...
  GPCOMPLETED   = 1
  GPERROR       = 2

  TravelDeadZone = 4  'The deadzone for the analog input from the remote
  cTravelDeadZone =  4          'The deadzone for the analog input from the remote

  SOUND_PIN = 7                            

{{ The following functions are the standard methods that is called by the main phoenix code.
All Control classes need to implement these...
}}
pub HexOn
  '' Returns the currrent HexOn State
  return _fhexOn
    
pub BodyPosX
  '' Returns the requested X Body position 
  return _BodyPosX
  
pub BodyPosY
  '' Returns the requested Y Body position 
  return _BodyPosY

pub BodyPosZ
  '' Returns the requested Z Body position 
  return _BodyPosZ

pub BodyRotX1
  return _BodyRotX1
  
pub BodyRotY1
  return _BodyRotY1
  
pub BodyRotZ1
  return _BodyRotZ1
  
pub TravelLengthX
  return _TravelLengthX
  
pub TravelLengthZ
  return _TravelLengthZ
  
pub TravelRotationY
  return _TravelRotationY
pub BalanceMode
  return _BalanceMode
pub fSLHold
  return _fSLHold

pub GaitType
  return _GaitType

pub LegLiftHeight
  return _LegLiftHeight
    
pub GPSeq
  return _GPSeq

pub GPStart  
  return _GPStart       ' should we start


pub SelectedLeg
  return _SelectedLeg
  
pub SLLegX
  return  _SLLegX
  
pub SLLegZ
  return _SLLegZ

pub SLLegY
  return _SLLegY

pub InputTimeDelay
  return _InputTimeDelay

pub SpeedControl
  return _SpeedControl



PUB Init(GPEnable)
'' This is the main Init function that is called by the Phoenix code to initialize
'' the controller. We are passed in information about if this phoenix support
'' General purpose sequences.
  _fGPEnable := GPEnable
  _fHexOn := 0                     ' Assume the Hex is logically off to start off with...
  _GaitType := 0                   ' Start off assuming Gait 0...

  DBG.Str(string("PS2 Init",13))
  PS2.Start(CFG#PS2_FIRSTPIN, 20_000) 'first_pin 0, Poll every 5ms  (1ms works for MadCatz wired but not for Predator wireless)


  'Body Positions
  _BodyPosX := 0
  _BodyPosY := 0
  _BodyPosZ := 0
      
  'Body Rotations
  _BodyRotX1 := 0
  _BodyRotY1 := 0
  _BodyRotZ1 := 0

   
  _BodyYOffset := 0
  _BodyYShift := 0
  _LegLiftHeight := 50
  _BalanceMode := 0
  _SelectedLeg := 255 ' No Leg selected
  bControlMode := WALKMODE
   
  DoubleHeightOn := false
  DoubleTravelOn := false

  return true 

PUB   AllowInterrupts(fAllow)    ' If on PS2 on hserial tell hserial to not processess...

  ' nothing for now

PUB ClearTravelData
    _TravelLengthX:=0
    _TravelLengthZ:=0
    _TravelRotationY:=0
 


PUB Input | wGPSeqPtr, w

 
  ' Then try to receive a packet of information from the PS2.
  if PS2.read_gamepad
   
    ' Lets see if the user pressed the start button. 
    if PS2.ButtonPressed(PS2#PSB_START)
      if(_fHexOn) 
                                'Turn off
         _BodyPosX := 0
         _BodyPosY := 0
         _BodyPosZ := 0
         _BodyRotX1 := 0
         _BodyRotY1 := 0
         _BodyRotZ1 := 0
         _TravelLengthX := 0
         _TravelLengthZ := 0
         _TravelRotationY := 0
         _BodyYOffset := 0
         _BodyYShift := 0
         _SelectedLeg := 255
                      
         DBG.str(string("Hex Off",13))
'         PS2.PlaySounds(@SND_HEXOFF)
         MSound (@SND_HEXOFF)
         _fHexOn := 0
      else 
        'Turn on
         DBG.str(string("Hex On - PS2",13))
'          PS2.PlaySounds(@SND_HEXON)
         MSound (@SND_HEXON)
         _fHexOn := 1

    if _fHexOn
      ' Translate mode
      if PS2.ButtonPressed(PS2#PSB_L1) ' check L1 button
        if bControlMode <> TRANSLATEMODE
           bControlMode := TRANSLATEMODE
           MSound (@SND_BEEP)
           DBG.str(string("Translate",13))
        elseif _SelectedLeg == 255
           bControlMode := WALKMODE
           MSound (@SND_BEEP2)
           DBG.str(string("Walk",13))
        else
           bControlMode := SINGLELEGMODE        

      ' Rotate mode     
      if PS2.ButtonPressed(PS2#PSB_L2) ' L2 button
         if bControlMode <> ROTATEMODE
           DBG.str(string("Rotate",13))
           MSound (@SND_BEEP)
           bControlMode := ROTATEMODE
         elseif _SelectedLeg == 255
           DBG.str(string("Walk",13))
           MSound (@SND_BEEP2)
           bControlMode := WALKMODE
         else
           bControlMode := SINGLELEGMODE        

      ' Single Leg Mode   
      if PS2.ButtonPressed(PS2#PSB_CIRCLE) ' Circle button test
         MSound (@SND_BEEP)
         if bControlMode<>SINGLELEGMODE 
             DBG.str(string("Single Leg",13))
             bControlMode := SINGLELEGMODE        
             if (_SelectedLeg==255) ' none
                _SelectedLeg := 0                            ' could choose later how to get the right leg
         else 
             bControlMode := WALKMODE
             MSound (@SND_BEEP2)
             _SelectedLeg := 255  'none

      ' GP Player
      if PS2.ButtonPressed(PS2#PSB_CROSS) ' Cross button
      
        if (_fGPEnable)    'F Button GP Player Mode Mode on/off -- SSC supports this mode
            if bControlMode <> GPPLAYERMODE
               _BodyPosX := 0
               _BodyPosZ := 0
               _BodyRotX1 := 0
               _BodyRotY1 := 0
               _BodyRotZ1 := 0
               _TravelLengthX := 0
               _TravelLengthZ := 0
               _TravelRotationY := 0
                     
               _SelectedLeg := 255  'none
               _fSLHold := 0
                   
               MSound (@SND_BEEP)
               bControlMode := GPPLAYERMODE
            else
               MSound (@SND_BEEP2)
               bControlMode := WALKMODE
        else
            MSound (@SND_BEEP2)

      '
      ' Common settings      
      if PS2.ButtonPressed(PS2#PSB_SQUARE) 'Square - Balance mode
        if not _BalanceMode 
           MSound (@SND_BEEP)
          _BalanceMode := 1
        else
          MSound (@SND_BEEP2)
         _BalanceMode := 0
    
      if PS2.ButtonPressed(PS2#PSB_TRIANGLE)  ' Stand up or sit down...
        if _BodyYOffset > 0
            _BodyYOffset := 0
        else
            _BodyYOffset := 35

      if PS2.ButtonPressed(PS2#PSB_PAD_UP)  ' ..
         MSound (@SND_BEEP)
          _BodyYOffset += 10
                
      if PS2.ButtonPressed(PS2#PSB_PAD_DOWN)  ' ..
         MSound (@SND_BEEP)
          _BodyYOffset -= 10
        
      if PS2.ButtonPressed(PS2#PSB_PAD_RIGHT)  ' ..
          IF _SpeedControl>0
             MSound (@SND_BEEP)
            _SpeedControl -=  50
          else
             MSound (@SND_BEEP2)

        
      if PS2.ButtonPressed(PS2#PSB_PAD_LEFT)  ' ..
          IF _SpeedControl<2000
             MSound (@SND_BEEP)
            _SpeedControl += 50
          else
             MSound (@SND_BEEP2)

        
      if bControlMode == WALKMODE 
          if PS2.ButtonPressed(PS2#PSB_SELECT) and ||(_TravelLengthX)<cTravelDeadZone and  ||(_TravelLengthZ)<cTravelDeadZone and ||(_TravelRotationY*2)<cTravelDeadZone
              'Switch gait
              if _GaitType < 7
                MSound (@SND_BEEP)
                _GaitType++
              else
                 MSound (@SND_BEEP2)
                  _GaitType := 0
                  
          if PS2.ButtonPressed(PS2#PSB_R1)
              DoubleHeightOn := not DoubleHeightOn
              IF DoubleHeightOn
                  _LegLiftHeight := 80          
              ELSE                  
                  _LegLiftHeight := 50

          if PS2.ButtonPressed(PS2#PSB_R2)
              DoubleTravelOn := not DoubleHeightOn

          if PS2.ButtonPressed(PS2#PSB_R3)
              WalkMethod := not WalkMethod

          if WalkMethod
              _TravelLengthZ := PS2.Analog(PS2#PSS_RY) - 128 'Right Stick Up/Down     
          else    
              _TravelLengthX := -(PS2.Analog(PS2#PSS_LX) - 128)
              _TravelLengthZ := (PS2.Analog(PS2#PSS_LY) - 128)

          IF not DoubleTravelOn
              _TravelLengthX := _TravelLengthX/2
              _TravelLengthZ := _TravelLengthZ/2
                
          _TravelRotationY := -(PS2.Analog(PS2#PSS_RX) - 128)/4  ';Right Stick Left/Right  
    
      elseif bControlMode == TRANSLATEMODE
          _BodyPosX := (PS2.Analog(PS2#PSS_LX) - 128)/2
          _BodyPosZ := -(PS2.Analog(PS2#PSS_LY) - 128)/2
          _BodyRotY1 := (PS2.Analog(PS2#PSS_RX) - 128)*2
          _BodyYShift := (-(PS2.Analog(PS2#PSS_RY) - 128)/2)

      elseif bControlMode == ROTATEMODE
          _BodyRotX1 := (PS2.Analog(PS2#PSS_LY) - 128)*2
          _BodyRotY1 := (PS2.Analog(PS2#PSS_RX) - 128)*2
          _BodyRotZ1 := (PS2.Analog(PS2#PSS_LX) - 128)*2
          _BodyYShift := (-(PS2.Analog(PS2#PSS_RY) - 128)/2)

          
       'Switch single leg
      elseif (bControlMode==SINGLELEGMODE) 
          if PS2.ButtonPressed(PS2#PSB_SELECT) 
              'Change Legs
              if _SelectedLeg < 5
                _SelectedLeg++
              else
                  _SelectedLeg  := 0

          'Single Leg Mode
          _SLLegX := ((PS2.Analog(PS2#PSS_LX)-128)/2) 
          _SLLegY := (PS2.Analog(PS2#PSS_RY)-128)/10
          _SLLegZ := (PS2.Analog(PS2#PSS_LY)-128)/2

          if PS2.ButtonPressed(PS2#PSB_R2)
              _fSLHold := not _fSLHold
      elseif (bControlMode == GPPLAYERMODE)
          if PS2.ButtonPressed(PS2#PSB_SELECT) 
              if (_GPStart == 0)
                  if _GPSeq < 5
                      _GPSeq++
                  else
                      _GPSeq := 0
          if PS2.ButtonPressed(PS2#PSB_R2)
              _GPStart := 1

    'Calculate walking time delay
    _InputTimeDelay := 128 -   (( ||(PS2.Analog(PS2#PSS_LX)-128) #> ||(PS2.Analog(PS2#PSS_LY)-128)) #>  ||(PS2.Analog(PS2#PSS_RX)-128))

  _BodyPosY := (_BodyYOffset + _BodyYShift) #> 0
return

pub GPSetState (state)
    case state
        GPERROR:
           MSound (@SND_BEEP2)
          _GPStart := 0         ' clear out the state  
        GPSTARTED:
'          PS2.OutputString(STRING("Start Sequence"))
        GPCOMPLETED:
'          PS2.OutputString(STRING("Completed"))
          _GPStart := 0         ' clear out the state  

pub MSound(dataptr) | cNotes
    cNotes := byte[dataptr++]
    repeat cNotes
      beep(SOUND_PIN, byte[dataptr], byte[dataptr + 1] * 25)
      dataptr +=2 


PUB beep(pinn, durr, freqq)
    if durr <> 0                                                       
      durr--
      ctrb[30..26] := %00100
      ctrb[8..0] := pinn
      frqb := calcfrq(freqq)
      if durr <> -2
        phsb := 0
      dira[pinn]~~                                            
      if durr <> -2
        repeat durr * 6000/16 ''''until phsa => durr ??? Not sure about timing, but works...
        'repeat (durr*freqq)/100
        waitpeq(0, |< pinn, 0)                                        
        ctrb := 0
        ctrb := frqb := 0
        durr := 0             

pri calcfrq(f)

  repeat 33
    result <<= 1
    if f => clkfreq
      f -= clkfreq
      result++
    f <<= 1
            