{{
Project Lynxmotion Phoenix
Description: Phoenix, control file.
  The control input subroutine for the phoenix software is placed in this file.
  Can be used with V2.0 and above
Configuration version: V1.0
Hardware setup: DIY RC receiver.


Date: Nov 17, 2010

NEW IN V1.0
 - First Release

RC Controller - Will depend on what capabilities different RC receivers use.


}}

OBJ
  RCV   : "ServoInput"          ' This is the simple function that will keep looking at the RC Pulses       
  DBG   : "SerialMirror"
  CFG   : "Phoenix_Config"                                     ' change this to the one specific for your setup.

con
  WALKMODE      =    0
  TRANSLATEMODE =    1
  ROTATEMODE    =    2
  SINGLELEGMODE =    3
  GPPLAYERMODE  =    4

  GPSTARTED     = 0    ' Used in the GPSetState function...
  GPCOMPLETED   = 1
  GPERROR       = 2

  PWAI_NUMPINS  = 6
  PWAI_RLR      = 0
  PWAI_RUD      = 1
  PWAI_LUD      = 2
  PWAI_LLR      = 3
  PWAI_GEAR     = 4             ' Geaer pin 2 settings on Hitec 6
  PWAI_AUX      = 5             ' Aux switch 

  MAXPW         = 2000
  MINPW         = 1000
  TravelDeadZone = 10  'The deadzone for the analog input from the remote


  MAX_RC_FAILURE = 8

  SOUND_PIN = 7                            

VAR
'=============================================================================
' Global - Local to this file only...
'=============================================================================

  ' Lets define the variables that we will set as part of the control object
  byte _fHexOn                  ' Is the Robot supposed to be on/Active?   

  byte _GaitType                 ' Wich Gait is active
  byte _BalanceMode              ' Balance mode
  word _InputTimeDelay           ' Desired Input Time delay generated from the inputs...

' Settings for position and rotation of body 
  long _BodyPosX
  long _BodyPosY
  long _BodyPosZ
  long _BodyRotX1
  long _BodyRotY1
  long _BodyRotZ1
' Settings for how far to move or rotate and left the leg.
  long _TravelLengthX
  long _TravelLengthZ
  long _TravelRotationY
  word _LegLiftHeight

' GP Player settings... Will need info as well from main object...
  byte  _GPSeq         ' Which Sequence
  byte  _GPStart       ' should we start
  byte _fGPEnable      ' need to set as part of the init...

' SingleLegMode

  byte  _SelectedLeg     
  byte _fSLHold
  long _SLLegX
  long _SLLegZ
  long _SLLegY
  
' Not sure about these...
  word    _BodyYOffset

' Non exported values...

  byte    bControlMode       ' Our current control mode...  
  byte   _fSwitchOnPrev
  byte   _fSwitchOn
  byte   _fInitOffsets           ' Have we init the centers yet?
  byte   _cRCFailure             ' how many times in a row have we failed?

' Normalized RC values
  long  alNPW[PWAI_NUMPINS]
  long  alPWOffsets[PWAI_NUMPINS]


DAT
        ' Wish there was a way to define an array here using a CON value for size...
        rcPins          LONG CFG#RC_PIN0, CFG#RC_PIN1, CFG#RC_PIN2, CFG#RC_PIN3, CFG#RC_PIN4, CFG#RC_PIN5
        alPW            LONG 1, 1, 1, 1, 1, 1

        SND_BEEP        byte    1, 50, 2000/25
        SND_BEEP2       byte    2, 50, 2000/25, 50, 2500/25
        SND_HEXON       byte    3, 60, 2000/25, 80, 2250/25, 100, 2500/25
        SND_HEXOFF      byte    3, 60, 2500/25, 80, 2250/25, 100, 1000/25

pub HexOn
  '' Returns the currrent HexOn State
  return _fhexOn
    
pub BodyPosX
  '' Returns the requested X Body position 
  return _BodyPosX
  
pub BodyPosY
  '' Returns the requested Y Body position 
  return _BodyPosY

pub BodyPosZ
  '' Returns the requested Z Body position 
  return _BodyPosZ

pub BodyRotX1
  return _BodyRotX1
  
pub BodyRotY1
  return _BodyRotY1
  
pub BodyRotZ1
  return _BodyRotZ1
  
pub TravelLengthX
  return _TravelLengthX
  
pub TravelLengthZ
  return _TravelLengthZ
  
pub TravelRotationY
  return _TravelRotationY
pub BalanceMode
  return _BalanceMode
pub fSLHold
  return _fSLHold

pub GaitType
  return _GaitType

pub LegLiftHeight
  return _LegLiftHeight
    
pub GPSeq
  return _GPSeq

pub GPStart  
  return _GPStart       ' should we start


pub SelectedLeg
  return _SelectedLeg
  
pub SLLegX
  return  _SLLegX
  
pub SLLegZ
  return _SLLegZ

pub SLLegY
  return _SLLegY

pub InputTimeDelay
  return _InputTimeDelay

pub SpeedControl
  return 0


PUB Init(GPEnable)

  _fGPEnable := GPEnable
  _fHexOn := 0                     ' Assume the Hex is logically off to start off with...
  _GaitType := 0                   ' Start off assuming Gait 0...

  DBG.Str(string("RC Controller Init",13))
  ' BUGBUG: should somehow move some of these defines to config file...
  RCV.start(@rcPins,6,@alPW)


  'Body Positions
  _BodyPosX := 0
  _BodyPosY := 0
  _BodyPosZ := 0
      
  'Body Rotations
  _BodyRotX1 := 0
  _BodyRotY1 := 0
  _BodyRotZ1 := 0

   
  _BodyYOffset := 0
  _LegLiftHeight := 50
  _BalanceMode := 0
  _SelectedLeg := 255 ' No Leg selected
  bControlMode := WALKMODE
  _cRCFailure := 0
  _fInitOffsets := 0
   
  return true 

PUB   AllowInterrupts(fAllow)    ' If on DIY on hserial tell hserial to not processess...

  ' nothing for now

PUB ClearTravelData
    _TravelLengthX:=0
    _TravelLengthZ:=0
    _TravelRotationY:=0
 


PUB Input | i, fRCValid 
 
  ' The ServoInput code runs in the background simply runs in the background and fils in an array.  We will first
  ' validate that the data is valid before we use it.  If not we will assume that we lost contact with the controller and turn
  ' the hex logically off.
  fRCValid := true              ' start off assuming that we are receiving valid RC Data.
  repeat i from 0 to PWAI_NUMPINS-1
     if alPW[i] =< MINPW or alPW[i] => MAXPW
        fRCValid := false
     else
        alNPW[i] := alPW[i] + alPWOffsets[i]
        if (i == PWAI_RLR) or (i == PWAI_RUD) or (i == PWAI_LLR)
            if ||(alNPW[i]-1500) < TravelDeadZone
                alNPW[i] := 1500 


  
  if fRCValid
      _cRCFailure := 0
      if not _fHexOn            ' Let the user know if this is our first valid data.
         ' See if we have initialized the Offsets yet....
         if  _fInitOffsets < 8
             repeat i from 0 to PWAI_NUMPINS-1
                 alPWOffsets[i] += alPW[i]
             _fInitOffsets++
             if _fInitOffsets == 8
                repeat i from 0 to PWAI_NUMPINS-1 
                   if (i == PWAI_RLR) or (i == PWAI_RUD) or (i == PWAI_LLR)
                       alPWOffsets[i] := 1500 - (alPWOffsets[i]>>3)
                   else  
                       alPWOffsets[i] := 0
             return  ' don't process until we have a complete set...          
                 
        'Turn on
         DBG.str(string("Hex On",13))
         MSound (@SND_HEXON)
         _fHexOn := 1

  ' Lets see what state we should be in.
      if alNPW[PWAI_AUX] <  1300 
          if bControlMode <> WalkMode 
              MSound(@SND_BEEP)
              DBG.Str(string("Walk Mode",13))
              bControlMode := WalkMode
      elseif alNPW[PWAI_AUX] < 1500
          if bControlMode <> TranslateMode
              DBG.Str(string("Translate Mode",13))
              MSound(@SND_BEEP)
              bControlMode := TranslateMode
      elseif alNPW[PWAI_AUX] < 1700      
          if bControlMode <> RotateMode
              MSound(@SND_BEEP)
              DBG.Str(string("Rotate Mode",13))
              bControlMode := RotateMode
      else
          if bControlMode <> SingleLegMode
              DBG.Str(string("Single Leg Mode",13))
              bControlMode := SingleLegMode        
              MSound(@SND_BEEP)
              _SelectedLeg := 0
      
          ' Note Have not done anything here yet for running sequences...
          ' likewise the ability to turn balance on or off...


      _fSwitchOnPrev := _fSwitchOn
      _fSwitchOn := (alNPW[PWAI_GEAR] > 1500)

      if _fSwitchOn <> _fSwitchOnPrev                   ' the switch toggled.
          if bControlMode == WalkMode                   '    '1-8 Button Gait select   
              if _GaitType < 7
                  _GaitType +=  1 
                  MSound(@SND_BEEP)
              else
                  _GaitType := 0
                  MSound(@SND_BEEP2)
          elseif bControlMode == SinglelegMode          ' use the toggle to select which leg to move
              if ~_SelectedLeg < 5
                  _SelectedLeg :=  ~_SelectedLeg + 1
                  MSound(@SND_BEEP)
              else
                  _SelectedLeg := 255
                  MSound(@SND_BEEP2)
 
                        
      'Body Height
      'BodyPosY = bPacket(PKT_LJOYUD) / 2
      _BodyPosy := ((alNPW[PWAI_LUD] #> 1100) - 1100) / 6
       
      'Leg lift height - Right slider has value 0-255 translate to 30-93
      'LegLiftHeight = 30 + bPacket(PKT_RSLIDER)/5
      _LegLiftHeight := 60                              ' no control of this yet...
       
      'Walk mode                      
      if bControlMode == WalkMode                  
          _TravelLengthX := -(alNPW[PWAI_RLR] - 1500) / 3
          _TravelLengthZ := -(alNPW[PWAI_RUD] - 1500) / 3   
          _TravelRotationY := -(alNPW[PWAI_LLR] - 1500)/10
       
      'Body move      
      elseif bControlMode == TranslateMode     
          _BodyPosX := (alNPW[PWAI_RLR]-1500)/6
          _BodyPosZ := (alNPW[PWAI_RUD]-1500)/6
          _BodyRotY1 := (alNPW[PWAI_LLR]-1500)/2
       
      'Body rotate    
      elseif bControlMode == RotateMode        
          _BodyRotX1 := (alNPW[PWAI_RUD]-1500)/2
          _BodyRotY1 := (alNPW[PWAI_LLR]-1500)/2
          _BodyRotZ1 := -(alNPW[PWAI_RLR]-1500)/2
       
      'Single Leg Mode
      elseif bControlMode == SingleLegMode
          _SLLegX := (alNPW[PWAI_RLR]-1500) / 3
          _SLLegZ := -(alNPW[PWAI_RUD]-1500) / 3
          _SLLegY := -(alNPW[PWAI_LLR]-1500)/ 10
       
      'Calculate walking time delay
      _InputTimeDelay := 128 - (||((alNPW[PWAI_RLR]-1500)/6) #> ||(((alNPW[PWAI_RUD]-1500)/6))) #> ||(((alNPW[PWAI_LLR]-1500)/6))
       


  else  
      ' Not valid data, let user know that and get back to the Off state...
      if _fHexOn
         _cRCFailure++
         if _cRCFailure => MAX_RC_FAILURE
        'Turn off
             _BodyPosX := 0
             _BodyPosY := 0
             _BodyPosZ := 0
             _BodyRotX1 := 0
             _BodyRotY1 := 0
             _BodyRotZ1 := 0
             _TravelLengthX := 0
             _TravelLengthZ := 0
             _TravelRotationY := 0
             _BodyYOffset := 0
             _SelectedLeg := 255
                                             
             DBG.str(string("Hex Off",13))
             MSound(@SND_HEXOFF)
             _fHexOn := 0
                                 

return

pub GPSetState (state)
    case state
        GPERROR:
          _GPStart := 0         ' clear out the state  
        GPSTARTED:
        GPCOMPLETED:
          _GPStart := 0         ' clear out the state  

pub MSound(dataptr) | cNotes
    cNotes := byte[dataptr++]
    repeat cNotes
      beep(SOUND_PIN, byte[dataptr], byte[dataptr + 1] * 25)
      dataptr +=2 


PUB beep(pinn, durr, freqq)
    if durr <> 0                                                       
      durr--
      ctrb[30..26] := %00100
      ctrb[8..0] := pinn
      frqb := calcfrq(freqq)
      if durr <> -2
        phsb := 0
      dira[pinn]~~                                            
      if durr <> -2
        repeat durr * 6000/16 ''''until phsa => durr ??? Not sure about timing, but works...
        'repeat (durr*freqq)/100
        waitpeq(0, |< pinn, 0)                                        
        ctrb := 0
        ctrb := frqb := 0
        durr := 0             

pri calcfrq(f)

  repeat 33
    result <<= 1
    if f => clkfreq
      f -= clkfreq
      result++
    f <<= 1        