{{
*******************************************************************************
* Ported to Propeller by KurtE
*******************************************************************************
Project Lynxmotion Phoenix
Description: Phoenix software
Software version: V2.0
Date: 29-10-2009
Programmer: Jeroen Janssen (aka Xan)

This is the servo driver for the phoenix code that uses the SSC-32 as the driver

}}

OBJ 
  DBG : "SerialMirror"  
  SSC32 : "FullDuplexSerial"  
  CFG   : "Phoenix_Config"                    ' change this to the one specific for your setup.

CON 
    _clkmode = xtal1 + pll16x      '80MHz operating frequency.
    _xinfreq = 5_000_000                      

'--------------------------------------------------------------------

DAT    

'//SSC Pin numbers
        cCoxaPin byte CFG#cRRCoxaPin,  CFG#cRMCoxaPin,  CFG#cRFCoxaPin, CFG#cLRCoxaPin, CFG#cLMCoxaPin, CFG#cLFCoxaPin
        cFemurPin byte  CFG#cRRFemurPin, CFG#cRMFemurPin, CFG#cRFFemurPin, CFG#cLRFemurPin, CFG#cLMFemurPin, CFG#cLFFemurPin
        cTibiaPin byte CFG#cRRTibiaPin, CFG#cRMTibiaPin, CFG#cRFTibiaPin, CFG#cLRTibiaPin, CFG#cLMTibiaPin, CFG#cLFTibiaPin
         


VAR   '' Global Variables and Cog Stack Space
  BYTE buffer[40] 'buffer to assemble output strings
  
'====================================================================
'--------------------------------------------------------------------
'[GP PLAYER]
'--------------------------------------------------------------------
'--------------------------------------------------------------------
'[VARIABLES]
   
'=============================================================================
' Initialise the software
'=============================================================================
PUB Init

'Checks SSC version number if it ends with "GP"
'enable the GP player if it does
    
  SSC32.start(CFG#cSSC_RXPIN ,CFG#cSSC_TXPIN, 0, CFG#cSSC_BAUD)     ' rx, tx, mode, baud Initialize a communication object that we will use to communicate with SSC-32
  DBG.str(string("Phoenix SSC32 Init", 13))

  return  CheckSequencesSupported


'--------------------------------------------------------------------
'[GP PLAYER]
'--------------------------------------------------------------------
pub CheckSequencesSupported | cbRead, iT
'    byte abVer[40]        ' give a nice large buffer.

    SSC32.str(string("ver",13))
    
    DBG.STR(string("Before SSC Read",13))
    cbRead := SSCRead(@buffer, 40, 100, 13)
    DBG.STR(string("After SSC Read", 13))

    result  := ((cbRead > 3) and (buffer[cbRead-3]=="G") and (buffer[cbRead-2]=="P") and (buffer[cbRead-1]==13))

    if (cbRead > 0) 
        repeat iT from 0 to cbRead -1
          DBG.hex(buffer[iT], 2)
        DBG.STR(string(" - "))
        repeat iT from 0 to cbRead -1 
          DBG.tx(buffer[iT])
        DBG.tx(13)
    DBG.str(string("Check GP Enable - cb "))
    DBG.dec(cbRead)
    DBG.str(string(" Enabled: "))
    DBG.dec(result)
    DBG.tx(13)



'--------------------------------------------------------------------
'[GP PLAYER]
'--------------------------------------------------------------------
'[GP PLAYER]
pub CheckSequence(seqnum) | abStat, cbRead
    
  'Start sequence
  DBG.str(string("GP Start: "))
  DBG.dec(seqnum)
  DBG.tx(13)
  ' First lets see if the sequence is defined.
  SSC32.str(string("EER -"))
  SSC32.dec(seqnum*2)
  SSC32.str(string(";2",13))
  cbRead := SSCRead(@buffer, 2, 1000, -1)  ' Read in 2 bytes to see if the specified sequence number exists
  if cbRead<>2 or ((buffer[0]==0 and buffer[1]==0) or (buffer[0]==$ff and buffer[1]==$ff))
      DBG.STR(string("Sequence Error",13))
      return false
  return true

pub RunSequence(seqnum) | abStat, cbRead
    
  SSC32.str(string("PL0SQ"))
  SSC32.dec(seqnum)
  SSC32.str(string("ONCE", 13))
   waitcnt(clkfreq/1000*20 + cnt)                       ' wait 20ms between calls
   
  'Wait for GPPlayer to complete sequence    
  repeat
      SSC32.str(string("QPL0", 13))
      cbRead := SSCRead(@abStat, 4, 100, -1)  '    [GPStatSeq, GPStatFromStep, GPStatToStep, GPStatTime]
       waitcnt(clkfreq/1000*20 + cnt)
  while ((cbRead == 4) and ((abStat.byte[0]<>255) or (abStat.byte[1]<>0) or (abStat.byte[2]<>0) or (abStat.byte[3]<>0)))
   
  return
   
'--------------------------------------------------------------------
' ServoDriverPin(legindex, CoxaA1, FemurA1, TibeaA1 )
'     called by main function to output 1 pin.  This is assumed to
'     be in a normalized angle that we will convert into the appropriate
'     values for the underlying servo driver...
'--------------------------------------------------------------------
pub OutputPin(LegIndex, CoxaA1, FemurA1, TibeaA1) | wCoxaSSCV, wFemurSSCV, wTibiaSSCV

    ' convert values to SSC32 values...
    wCoxaSSCV := CoxaA1*1000/1059+650
    wFemurSSCV := FemurA1*1000/1059+650
    wTibiaSSCV := TibeaA1*1000/1059+650
    SSC32.tx(cCoxaPin[LegIndex]  + $80)
    SSC32.tx(wCoxaSSCV >> 8)
    SSC32.tx(wCoxaSSCV & $ff)
    SSC32.tx(cFemurPin[LegIndex] + $80)
    SSC32.tx(wFemurSSCV >> 8)
    SSC32.tx(wFemurSSCV & $ff)
    SSC32.tx(cTibiaPin[LegIndex] + $80)
    SSC32.tx(wTibiaSSCV >> 8)
    SSC32.tx(wTibiaSSCV & $ff)

'--------------------------------------------------------------------
'[SERVO DRIVER commit] We are readdy to commit the changes...
'         as much of the command as we can without committing it.  This
'         allows us to once the previous update was completed to quickly 
'        get the next command to start
'--------------------------------------------------------------------
pub Commit (SSCTime)
  SSC32.tx($A1)
  SSC32.tx(SSCTime >> 8)
  SSC32.tx(SSCTime & $ff)

  
pub FreeServos | LegIndex
    repeat LegIndex from  0 to 32
      SSC32.tx("#")
      SSC32.dec(LegIndex)
      SSC32.tx("P")
      SSC32.tx("0")

    SSC32.str(string("T200", 13))




' Quick and dirty helper function to read so many bytes in from the SSC with a timeout and an end of character marker...
pri SSCRead (pb, cb, wTimeout, wEOL) | ich, pbIn

  pbIn := pb
  repeat cb
    ich := SSC32.rxtime(wTimeout)

    if ich == -1
      return (pb-pbIn)

    byte[pb++] := ich

    if ich == wEOL
        return (pb-pbIn)    ' we matched so get out of here.

  return (cb)